@@ -1,6 +1,6 @@
 /*
 This is a Optical-Character-Recognition program
-Copyright (C) 2000-2006 Joerg Schulenburg
+Copyright (C) 2000-2007 Joerg Schulenburg
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -72,7 +72,11 @@ int load_db(void) {
     pp = (pix *)malloc(sizeof(pix));
     if( !pp ) fprintf(stderr,"malloc error in load_db pix\n");
 
-    readpgm(s2, pp, 0 * JOB->cfg.verbose);
+    // if (JOB->cfg.verbose) fprintf(stderr,"\n# readpgm %s ",s2);
+    if (readpgm(s2, pp, 0 * JOB->cfg.verbose)!=0) {
+      fprintf(stderr,"\ndatabase error: readpgm %s\n", s2);
+      exit(-1);
+    }
 
     box1 = (struct box *)malloc_box(NULL);
     if(!box1) fprintf(stderr,"malloc error in load_db box1\n");
