@@ -1,6 +1,6 @@
 /*
 This is a Optical-Character-Recognition program
-Copyright (C) 2000-2006 Joerg Schulenburg
+Copyright (C) 2000-2007 Joerg Schulenburg
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -55,9 +55,10 @@ int detect_lines1(pix * p, int x0, int y0, int dx, int dy)
 {
   int i, jj, j2, y, yy, my, mi, mc, i1, i2, i3, i4,
       m1, m2, m3, m4, ma1, ma2, ma3, ma4, m3pre, m4pre;
-  struct box *box2;
+  struct box *box2, *box3;  /* box3 is for verbose / debugging */
   struct tlines *lines = &JOB->res.lines;
 
+  /* ToDo: optional read line-data from external source??? */
   if (lines->num == 0) { // initialize one dummy-line for pictures etc.
       lines->m4[0] = 0;
       lines->m3[0] = 0;
@@ -73,17 +74,34 @@ int detect_lines1(pix * p, int x0, int y0, int dx, int dy)
   if (dy < 4)
     return 0;  /* image is to low for latin chars */
   my = jj = 0;
-  // get the mean value of height
+  // get the mean height of all hollow chars
+  // (better than mean value of everything including bg-pattern or dust?)
   for_each_data(&(JOB->res.boxlist)) {
     box2 = (struct box *)list_get_current(&(JOB->res.boxlist));
     if ( box2->c != PICTURE
-      && box2->y1 - box2->y0 + 1 >= 4   /* 4x6 font */
+      && box2->num_frames>1 && box2->num_frames<3 /* 1 or 2 holes */
       && box2->y0 >= y0 && box2->y1 <= y0 + dy
-      && box2->x0 >= x0 && box2->x1 <= x0 + dx ) {
+      && box2->x0 >= x0 && box2->x1 <= x0 + dx 
+      && box2->frame_vol[0]>0
+      && box2->frame_vol[1]<0
+      ) {
 	jj++;
 	my += box2->y1 - box2->y0 + 1;
     }
   } end_for_each(&(JOB->res.boxlist));
+  if (jj==0) {
+    // get the mean height of all chars
+    for_each_data(&(JOB->res.boxlist)) {
+      box2 = (struct box *)list_get_current(&(JOB->res.boxlist));
+      if ( box2->c != PICTURE
+        && box2->y1 - box2->y0 + 1 >= 4   /* 4x6 font */
+        && box2->y0 >= y0 && box2->y1 <= y0 + dy
+        && box2->x0 >= x0 && box2->x1 <= x0 + dx ) {
+          jj++;
+          my += box2->y1 - box2->y0 + 1;
+      }
+    } end_for_each(&(JOB->res.boxlist));
+  }
   if (jj == 0)
     return 0;  /* no chars detected */
     
@@ -95,7 +113,7 @@ int detect_lines1(pix * p, int x0, int y0, int dx, int dy)
    */
   my /= jj;          /* we only care about chars with high arround my */
   if (JOB->cfg.verbose & 16)
-    fprintf(stderr," detect_lines1(%d %d %d %d) chars=%d my=%d\n#  ",
+    fprintf(stderr,"\n# detect_lines1(%d %d %d %d) vvv&16 chars=%d my=%d\n#  ",
       x0, y0, dx, dy, jj, my);
   // "my" is the average over the whole image (bad, if different fontsizes)
 
@@ -115,29 +133,36 @@ int detect_lines1(pix * p, int x0, int y0, int dx, int dy)
       fprintf(stderr,"searching new line %d\n#  ",i /* lines->num */);
 #endif
 
+    box3 = NULL; /* mark the most upper box starting next line */
     // find highest point of next line => store to m1-min (m1>=y)
     // only objects greater 2/3*my and smaller 3*my are allowed
     // a higher "!" at end of line can result in a to low m1
     for_each_data(&(JOB->res.boxlist)) {
       box2 = (struct box *)list_get_current(&(JOB->res.boxlist));
       if (box2->line>0 || box2->c == PICTURE) continue;
-      yy = lines->dy * box2->x0 / (p->x); /* correct crooked lines */
+      if (lines->dx)
+        yy = lines->dy * box2->x0 / (lines->dx); /* correct crooked lines */
+      else yy=0;
       if (   box2->y0 >= y + yy && box2->y1 < y0 + dy	// lower than y
-	  && box2->x0 >= x0     && box2->x1 < x0 + dx	// in image box ?
+	  && box2->x0 >= x0     && box2->x1 < x0 + dx	// within box ?
 	  && box2->c != PICTURE	// no picture
-	  && box2->num_boxes<=1 // ignore 2 for "!?i" 3 for "&auml;"
+	  && box2->num_boxes <= 1 // ignore 2 for "!?i" 3 for "&auml;"
 	  && 3 * (box2->y1 - box2->y0) > 2 * my	// not to small
 	  &&     (box2->y1 - box2->y0) < 3 * my // not to big
 	  &&     (box2->y1 - box2->y0) > 4)     // minimum absolute size
        {
 	  if (box2->y0 < m1 + yy) {
 	    m1 = box2->y0 - yy;  /* highest upper boundary */
+	    box3 = box2;
 	  }
 	  // fprintf(stderr,"\n %3d %3d %+3d %d m1= %3d",
           //   box2->x0, box2->y0, box2->y1 - box2->y0 + 1, box2->num_boxes, m1);
        }
     } end_for_each(&(JOB->res.boxlist));
-    if (m1 >= y0+dy) break; /* no further line found */
+    if (!box3 || m1 >= y0+dy) break; /* no further line found */
+    if (JOB->cfg.verbose & 16)
+      fprintf(stderr," most upper box at new line xy= %4d %4d %+4d %+4d\n#  ",
+       box3->x0, box3->y0, box3->x1-box3->x0, box3->y1-box3->y0);
 
     // at the moment values depend from single chars, which can 
     //    result in bad values (ex: 4x6 /\=)
@@ -154,7 +179,8 @@ int detect_lines1(pix * p, int x0, int y0, int dx, int dy)
       if (box2->line>0 || box2->c == PICTURE) continue;
       if (   box2->y0 < y0 || box2->y1 >= y0 + dy
 	  || box2->x0 < x0 || box2->x1 >= x0 + dx ) continue; // out of image
-      yy = lines->dy * box2->x0 / (p->x);
+      if (lines->dx) yy = lines->dy * box2->x0 / (lines->dx);
+      else yy=0;
       /* check for ij-dots, used if chars of same high */
       if ( box2->y0 >= y + yy
 	&& box2->y0 >= y
@@ -174,11 +200,15 @@ int detect_lines1(pix * p, int x0, int y0, int dx, int dy)
         // ToDo: we need a better (local?) algorithm for big headlines > 2*my
         && box2->y1 <= m1 + yy + 3 * my
         && box2->y1 >= m1 + yy + my / 2
+        // lines can differ in high, my may be to small (smaller headlines)
+        && box2->y0+box2->y1 <= 2*box3->y1
          )
       {
           jj++;  // count chars for debugging purpose
 	  if (box2->y0 > m2 + yy) {
 	    m2 = box2->y0 - yy;  /* highest upper boundary */
+            if (JOB->cfg.verbose & 16)
+	      fprintf(stderr," set m2= %d yy= %d\n#  ",m2, yy);
 	  }
 	  if (box2->y1 > m4 + yy && (my>6 || box2->y1 < m3+my)) {
 	    m4 = box2->y1 - yy;  /* lowest lower boundary, small font lines can touch */
@@ -228,7 +258,7 @@ int detect_lines1(pix * p, int x0, int y0, int dx, int dy)
     for_each_data(&(JOB->res.boxlist)) {
       box2 = (struct box *)list_get_current(&(JOB->res.boxlist));
       if (box2->line>0 || box2->c == PICTURE) continue;
-      yy = lines->dy * box2->x0 / (p->x);
+      if (lines->dx) yy = lines->dy * box2->x0 / (lines->dx); else yy=0;
       if (box2->y0 >= y + yy && box2->y1 < y0 + dy	// lower than y
 	  && box2->x0 >= x0 && box2->x1 < x0 + dx	// in box ?
 	  && box2->c != PICTURE	// no picture
@@ -263,7 +293,7 @@ int detect_lines1(pix * p, int x0, int y0, int dx, int dy)
     for_each_data(&(JOB->res.boxlist)) {
       box2 = (struct box *)list_get_current(&(JOB->res.boxlist));
       if (box2->line>0 || box2->c == PICTURE) continue;
-      yy = lines->dy * box2->x0 / (p->x);
+      if (lines->dx) yy = lines->dy * box2->x0 / (lines->dx); else yy=0;
       if (   box2->y0 >= y0 && box2->y1 < y0 + dy
 	  && box2->x0 >= x0 && box2->x1 < x0 + dx	// in box ?
 	  && box2->c != PICTURE	// no picture
@@ -320,7 +350,7 @@ int detect_lines1(pix * p, int x0, int y0, int dx, int dy)
     if (jj&2)
       m4 = m3 + (m4 - m1) / 4 + 1;	/* chars have same lower base */
     if (jj>0 && JOB->cfg.verbose & 16) {
-      fprintf(stderr,"#     new: m=%4d %+2d %+2d %+2d  my=%4d\n#  ",
+      fprintf(stderr," m= %4d %+2d %+2d %+2d  my= %4d\n#  ",
               m1, m2-m1, m3-m1, m4-m1, my);
      }
 #endif
@@ -356,9 +386,9 @@ int detect_lines1(pix * p, int x0, int y0, int dx, int dy)
 // ----- layout analyzis of dx*dy region at x0,y0 -----
 // ----- detect lines via recursive division (new version) ---------------
 //   what about text in frames???
-//  
+//  ToDo: change to bottom-top analyse or/and take rotation into account
 int detect_lines2(pix *p,int x0,int y0,int dx,int dy,int r){
-    int i,x2,y2,x3,y3,x4,y4,x5,y5,y6,mx,my;
+    int i,x2,y2,x3,y3,x4,y4,x5,y5,y6,mx,my,x30,x31,y30,y31;
     struct box *box2,*box3;
     // shrink box
     if(dx<=0 || dy<=0) return 0;
@@ -379,16 +409,16 @@ int detect_lines2(pix *p,int x0,int y0,int dx,int dy,int r){
       if(box3->y0>=y0  && box3->y1<y0+dy &&
          box3->x0>=x0  && box3->x1<x0+dx)
       {
-    	if( box3->x1 > x3 ) x3=box3->x1;
-        if( box3->x0 < x2 ) x2=box3->x0;
-        if( box3->y1 > y3 ) y3=box3->y1;
-        if( box3->y0 < y2 ) y2=box3->y0;
+    	if( box3->x1 > x3 ) x3=box3->x1; // max x
+        if( box3->x0 < x2 ) x2=box3->x0; // min x
+        if( box3->y1 > y3 ) y3=box3->y1; // max y
+        if( box3->y0 < y2 ) y2=box3->y0; // min y
         if(box3->c!=PICTURE)
         if( box3->y1 - box3->y0 > 4 )
         {
           i++;
-      	  mx+=box3->x1-box3->x0+1;
-          my+=box3->y1-box3->y0+1;
+      	  mx+=box3->x1-box3->x0+1; // mean x
+          my+=box3->y1-box3->y0+1; // mean y
         }
       }
     } end_for_each(&(JOB->res.boxlist));
@@ -397,7 +427,7 @@ int detect_lines2(pix *p,int x0,int y0,int dx,int dy,int r){
 
     if(i==0 || dx<=0 || dy<=0) return 0;
     mx/=i;my/=i;
-    // better look for widest h/v-gap
+    // better look for widest h/v-gap, ToDo: vertical lines?
     if(r<8){ // max. depth
 
       // detect widest horizontal gap
@@ -413,7 +443,9 @@ int detect_lines2(pix *p,int x0,int y0,int dx,int dy,int r){
 
       	  y4=y0+dy-1;  // nearest vert. box
           x4=x0+dx-1;
-          // look for nearest nb of every box
+          // ToDo: rotate back box2->x1,y1 to x21,y21
+          // look for nearest lowest (y4) and right (x4) neighbour
+          //   of every box (box2)
 	  for_each_data(&(JOB->res.boxlist)) {
 	    box3 = (struct box *)list_get_current(&(JOB->res.boxlist));
 	    if(box3!=box2)
@@ -421,10 +453,17 @@ int detect_lines2(pix *p,int x0,int y0,int dx,int dy,int r){
             if(box3->x0>=x0  && box3->x1<x0+dx)
      	    if(box3->c!=PICTURE) /* ToDo: not sure, that this is a good idea */
        	    if(box3->y1-box3->y0>my/2 ){
-              if( box3->y1 > box2->y1  &&  box3->y0 < y4 ) y4=box3->y0-1;
-              if( box3->x1 > box2->x1  &&  box3->x0 < x4 ) x4=box3->x0-1;
+       	      // ToDo: here we need the rotation around box2
+       	      x30=box3->x0;
+       	      x31=box3->x1;
+       	      y30=box3->y0;
+       	      y31=box3->y1;
+       	      // get min. distances to lower and to right direction
+              if( y31 > box2->y1  &&  y30 < y4 ) y4=y30-1;
+              if( x31 > box2->x1  &&  x30 < x4 ) x4=x30-1;
             }
       	  } end_for_each(&(JOB->res.boxlist));
+      	  // set the witdht and position of largest hor./vert. gap
           // largest gap:          width           position
           if( y4-box2->y1 > y3 ) { y3=y4-box2->y1; y2=(y4+box2->y1)/2; }
           if( x4-box2->x1 > x3 ) { x3=x4-box2->x1; x2=(x4+box2->x1)/2; }
@@ -508,190 +547,146 @@ int my_sqrt(int x){
 }
 
 /*
- * search nearest neighbour of each box and average vectors
+** Detect rotation angle (one for whole image)
+** old: longest text-line and determining the angle of this line.
+ *
+ * search right nearest neighbour of each box and average vectors
  * to get the text orientation,
  * upside down decision is not made here (I dont know how to do it)
  *  ToDo: set job->res.lines.{dx,dy}
- * pass 1: get mean nearest char distance
- * pass 2: get mean nearest char distance to pass-1 vector
- * pass 3: go to farest char in pass-2 direction and averge (ToDo)
- *  or  check directions with most crossings? for each of outside chars
- *  ToDo: store distances in ranges to detect outriders ???
+ * pass 1: get mean vector to nearest char
+ * pass 2: get mean vector to nearest char without outriders to pass 1
+ * extimate direction as (dx,dy,num)[pass]
+ * ToDo: estimate an error, boxes only work fine for zero-rotation
+ *       for 45 degree use vectors, not boxes to get base line
  */
-int new_detect_rotation_angle(job_t *job, int *dx, int *dy){
-  struct box *box2, *box3, *box_nn;
+#define INorm 1024   /* integer unit 1.0 */
+int detect_rotation_angle(job_t *job){
+  struct box *box2, *box3,
+        *box_nn;  /* nearest neighbour box */
   int x2, y2, x3, y3, dist, mindist, pass,
+      rx=0, ry=0, re=0,  // final result
       /* to avoid 2nd run, wie store pairs in 2 different categories */
-      nn1=0, nn2=0, /* num_pairs within Intervals */ 
-      dx1=0, dy1=0,
-      dx2=0, dy2=0; /* borders of interval (dx,dy) */ 
+      nn[4]={0,0,0,0}, /* num_pairs used for estimation [(pass-1)%2,pass%2] */ 
+      dx[4]={0,0,0,0}, /* x-component of rotation vector per pass */
+      dy[4]={0,0,0,0}, /* y-component of rotation vector per pass */ 
+      er[4]={INorm/4,0,0,0}; /* mean angle deviation to pass-1 (radius^2) */
+      // de;         /* ToDo: absolute maximum error (dx^2+dy^2) */ 
+      // ToDo: next pass: go to bigger distances and reduce max error
+      // error is diff between passes? or diff of bottoms and top borders (?)
 
-  for (pass=1;pass<3;pass++) {
+  rx=1024; ry=0;  // default
+  for (pass=0;pass<4;pass++) {
     for_each_data(&(job->res.boxlist)) {
       box2 = (struct box *)list_get_current(&(job->res.boxlist));
       if (box2->c==PICTURE) continue;
-      x2 = (box2->x0 + box2->x1)/2;
-      y2 = (box2->y0 + box2->y1)/2; /* get middle point of the box */
       /* subfunction probability of char */
-      if  (box2->x1 - box2->x0 < 3) continue; /* smallest font is 4x6 */
-      if  (box2->y1 - box2->y0 < 5) continue;
+      // i?
+      // if  (box2->x1 - box2->x0 < 3) continue; /* smallest font is 4x6 */
+      if  (box2->y1 - box2->y0 < 4) continue;
       /* set maximum possible distance */
-      mindist = job->src.p.x * job->src.p.x + job->src.p.y * job->src.p.y;
-      box_nn=NULL;
+      box_nn=box2; // initial box to compare with
 
+      // ToDo: clustering or majority
+      // the algorithm is far from being perfect, pitfalls are likely
+      // but its better than the old algorithm, ToDo: database-rotated-images
+      mindist = job->src.p.x * job->src.p.x + job->src.p.y * job->src.p.y;
+      /* get middle point of the box */
+      x2 = (box2->x0 + box2->x1)/2;
+      y2 = (box2->y0 + box2->y1)/2;
+      re=0;
+      /* search for nearest neighbour box_nn[pass+1] of box_nn[pass] */
       for_each_data(&(job->res.boxlist)) {
         box3 = (struct box *)list_get_current(&(job->res.boxlist));
         /* try to select only potential neighbouring chars */
-        /* select out all senceless combinations */ 
+        /* select out all senseless combinations */ 
         if (box3->c==PICTURE || box3==box2) continue;
         x3 = (box3->x0 + box3->x1)/2;
         y3 = (box3->y0 + box3->y1)/2; /* get middle point of the box */
-        if (x3<x2) continue;  /* left to right */
-        /* neighbours should have same order of size */
-        if (2*(box3->y1-box3->y0) <   (box2->y1-box2->y0)) continue;
-        if (  (box3->y1-box3->y0) > 2*(box2->y1-box2->y0)) continue;
-        if (  (box3->x1-box3->x0) > 2*(box2->x1-box2->x0)) continue;
-        if (2*(box3->x1-box3->x0) <   (box2->x1-box2->x0)) continue;
-        /* should be in right range (ToDo: 2nd pass) */
-        if (pass==1) {
-          if ( 2*(box3->x0-box2->x1) > 3*(box2->x1-box2->x0) ) continue;
-          if ( 4*(box3->y0-box2->y1) > 1*(box2->y1-box2->y0) ) continue;
-          if ( 4*(box2->y0-box3->y1) > 1*(box2->y1-box2->y0) ) continue;
+        if (x3<x2) continue; /* simplify by going right only */
+        // through-away deviation of angles if > pass-1?
+        // scalprod max in direction, cross prod min in direction
+        //  a,b (vectors): <a,b>^2/(|a|*|b|)^2 = 0(90deg)..0.5(45deg).. 1(0deg)
+        //   * 1024 ??
+        if (pass>0) {  // new variant = scalar product
+          // danger of int overflow, ToDo: use int fraction
+          re =(int) ((1.*(x3-x2)*dx[pass-1]+(y3-y2)*dy[pass-1])
+              *(1.*(x3-x2)*dx[pass-1]+(y3-y2)*dy[pass-1])*INorm
+              /(1.*((x3-x2)*(x3-x2)+(y3-y2)*(y3-y2))
+               *(1.*dx[pass-1]*dx[pass-1]+dy[pass-1]*dy[pass-1])));
+          if (INorm-re>er[pass-1]) continue; // hits mean deviation
         }
+        /* neighbours should have same order of size (?) */
+        if (3*(box3->y1-box3->y0+4) < 2*(box2->y1-box2->y0+1)) continue;
+        if (2*(box3->y1-box3->y0+1) > 3*(box2->y1-box2->y0+4)) continue;
+        if (2*(box3->x1-box3->x0+1) > 5*(box2->x1-box2->x0+4)) continue;
+        if (5*(box3->x1-box3->x0+4) < 2*(box2->x1-box2->x0+1)) continue;
+        /* should be in right range, Idea: center3 outside box2? noholes */
+        if ((x3<box2->x1-1) && (x3>box2->x0+1)
+         && (y3<box2->y1-1) && (y3>box2->y0+1)) continue;
+        // if chars are of different size, connect careful 
+        if (  abs(x3-x2) > 2*(box2->x1 - box2->x0 + box3->x1 - box3 ->x0 + 2)) continue;
+        if (  abs(y3-y2) >   (box2->x1 - box2->x0 + box3->x1 - box3 ->x0 + 2)) continue;
         dist = (y3-y2)*(y3-y2) + (x3-x2)*(x3-x2);
-        if (dist<16) continue; /* minimum distance^2 is 4^2 */
-        if (pass!=1 && nn1>0) {
-          dist = (y3-y2-3*dy1/1024)*(y3-y2-3*dy1/1024)
-               + (x3-x2-3*dx1/1024)*(x3-x2-3*dx1/1024);
-          if (dist>(dx1*dx1+dy1*dy1)/(1024*1024)) continue;
-        }
-        if (dist<mindist) {mindist=dist; box_nn=box3;}
+        // make distances in pass-1 directions shorter or continue if not in pass-1 range?
+        if (dist<9) continue; /* minimum distance^2 is 3^2 */
+        if (dist<mindist) { mindist=dist; box_nn=box3;}
+        // fprintf(stderr,"x y %d %d  %d %d dist %d min %d\n",
+        //         x2,y2,x3,y3,dist,mindist);
       } end_for_each(&(job->res.boxlist));
       
-      if (!box_nn) continue; /* has no neighbour */
+      if (box_nn==box2) continue; /* has no neighbour, next box */
       
       box3=box_nn; dist=mindist;
       x3 = (box3->x0 + box3->x1)/2;
       y3 = (box3->y0 + box3->y1)/2; /* get middle point of the box */
       // dist = my_sqrt(1024*((x3-x2)*(x3-x2)+(y3-y2)*(y3-y2)));
-      dist=32;
-      if (pass==1) {
-        dx1+=(x3-x2)*1024*32/dist; /* normalized before averaging */
-        dy1+=(y3-y2)*1024*32/dist; /* 1024 is for the precision */
-        nn1++;
-       } else {
-        dx2+=(x3-x2)*1024*32/dist; /* normalized before averaging */
-        dy2+=(y3-y2)*1024*32/dist;
-        nn2++;
+      // compare with first box
+      x2 = (box2->x0 + box2->x1)/2;
+      y2 = (box2->y0 + box2->y1)/2;
+      // if the high of neighbouring boxes differ, use min diff (y0,y1)
+      if (pass>0 && 16*abs(dy[pass-1]) < dx[pass-1]) // dont work for strong rot.
+      if (abs(box2->y1-box2->y0-box3->y1+box3->y0)>(box2->y1-box2->y0)/8) {
+        // ad eh ck ...
+        if (abs(box2->y1-box3->y1)<abs(y3-y2)) { y2=box2->y1; y3=box3->y1; }
+        // ag ep qu ...
+        if (abs(box2->y0-box3->y0)<abs(y3-y2)) { y2=box2->y0; y3=box3->y0; }
+      }
+      if (abs(x3-x2)<4) continue;
+      dx[pass]+=(x3-x2)*1024; /* normalized before averaging */
+      dy[pass]+=(y3-y2)*1024; /* 1024 is for the precision */
+      nn[pass]++;
+      if (pass>0) { // set error = mean deviation from pass -1
+        re = INorm-(int)((1.*(x3-x2)*dx[pass-1]+(y3-y2)*dy[pass-1])
+                    *(1.*(x3-x2)*dx[pass-1]+(y3-y2)*dy[pass-1])*INorm
+                    /((1.*(x3-x2)*(x3-x2)+(y3-y2)*(y3-y2))
+                     *(1.*dx[pass-1]*dx[pass-1]+dy[pass-1]*dy[pass-1]))
+                    );
+        er[pass]+=re;
       }
 #if 0
       if(JOB->cfg.verbose)
-        fprintf(stderr,"# nn (%4d,%4d) %d\n", x3-x2,y3-y2,mindist);
-#endif
-
+        fprintf(stderr,"# next nb (x,y,dx,dy,re) %6d %6d %5d %5d %5d pass %d\n",
+                x2, y2, x3-x2, y3-y2, re, pass+1);
+#endif      
     } end_for_each(&(job->res.boxlist));
-    
-    if (pass==1 && nn1) {dx1/=nn1;dy1/=nn1;}  /* meanvalues */
-    if (pass!=1 && nn2) {dx2/=nn2;dy2/=nn2;}
-    if (!(nn1+nn2))  {*dx=1024;*dy=0;} /* set default value for horizontal */
-    else { if (nn2)  {*dx=dx2;*dy=dy2;}
-                else {*dx=dx1;*dy=dy1;} }
-
+    if (!nn[pass]) break;
+    if (nn[pass]) {
+      /* meanvalues */
+      rx=dx[pass]/=nn[pass];
+      ry=dy[pass]/=nn[pass];
+      if (pass>0) er[pass]/=nn[pass];
+    }
     if(JOB->cfg.verbose)
-      fprintf(stderr,"# rotation angle (x,y,num) (%d,%d,%d) (%d,%d,%d), pass %d\n",
-              dx1,dy1,nn1,dx2,dy2,nn2,pass);
+      fprintf(stderr,"# rotation angle (x,y,maxr,num)"
+                     " %6d %6d %6d %4d pass %d\n",
+              rx, ry, er[pass], nn[pass], pass+1);
   }
-  if (nn2>4 && abs(dy2*100)>abs(dx2*50))
+  if (abs(ry*100)>abs(rx*50))
     fprintf(stderr,"<!-- gocr will fail, strong rotation angle detected -->\n");
-  return (*dy)*((*dx)?job->src.p.x/(*dx):1);
-  /*
-   *   ToDo: 2nd pass? fine tuning, scan only for main direction
-   *      or follow nn-chain for every line (longest give best result)
-   *    - set probabilities of directions (unsure=50, sure=100)
-   */
-}
-
-/*
-** Detect rotation angle (one for whole image)
-** Do it by finding longest text-line and determining the angle of this line.
-** Angle is defined by dy (difference in y per width of image dx)
-*/
-int detect_rotation_angle(job_t *job){
-  // FIXME jb: remove pp
-  pix *pp = &job->src.p;
-  struct box *box2, *b3, *b4;
-  int i,x,y,dx,dy,j,y2,k,my,ty=0,ml,mr,il,ir;
-
-  /* this is a experimentel new clean routine which shall
-   * replace old detect_rotation_angle in future,
-   * it can detect angles between 0 and 90 degree
-   * (later argument: pointer to the interesting char)
-   */
-  dx=dy=0;
-  new_detect_rotation_angle(job,&dx,&dy); /* pass 1 */
-  dx=job->src.p.x; dy=0;
-
-  /* ToDo: check another idea: get 2 boxes with most boxes between them */
-
-  // ----- detect longest line, is it horizontal? ---------------
-  // use function and box ??? for boxes rotated by different angles???
-  {
-    x=0; y2=0;
-    for_each_data(&(job->res.boxlist)) {
-	box2 = (struct box *)list_get_current(&(job->res.boxlist));
-	if( box2->x1 - box2->x0 + 1 > x
-	 && box2->y1 - box2->y0 + 1 == 1 ){
-	   x = box2->x1 - box2->x0 - 1; y2=box2->y1; 
-	}
-    } end_for_each(&(job->res.boxlist));
-    if ( 2*x > pp->x && pp->x > 100 ){
-     if(JOB->cfg.verbose)fprintf(stderr, "# perfect flat line found, dy=0\n");
-     JOB->res.lines.ly=y2;
-     return 0;
-    }
-    b3=b4=NULL;
-    if(JOB->cfg.verbose)fprintf(stderr, "# detect longest line"); // or read from outside???
-    // most black/white changes ??? new: most chars
-    for(ty=y2=i=y=0;y<pp->y;y+=4){
-      my = j = il=ir=0;
-      for_each_data(&(job->res.boxlist)) {
-	box2 = (struct box *)list_get_current(&(job->res.boxlist));
-        if(box2->y0<=y && box2->y1>=y) {
-          if( box2->x1<  pp->x/4 ) il++;
-          if( box2->x0>3*pp->x/4 ) ir++;
-          j++; my+=box2->y1-box2->y0+1; 
-        }
-      } end_for_each(&(job->res.boxlist));
-      /* j=num_cross(0,pp->x-1,y,y,pp,cs); old slow version */
-      if( j>i && il>3 && ir>3 ){ i=j; y2=y; ty=my; } 
-    }
-    my=0;if (i>0) my=ty/i;
-    if(JOB->cfg.verbose)fprintf(stderr," - at y=%d crosses=%3d my=%d",y2,i,my);
-    // detect the base line
-    if(i>10){  // detect 2nd ??? highest base (not !?) of right/left side
-      i=il=ir=ml=mr=0;
-      for_each_data(&(job->res.boxlist)) {
-	box2 = (struct box *)list_get_current(&(job->res.boxlist));
-        if(box2->y0<=y2 && box2->y1>=y2 && box2->y1-box2->y0>2*my/3){
-          k=box2->y1; i++; y=box2->y1; // meanvalues and highest
-          if(box2->x1<  pp->x/4){ il++;ml+=y;if ( (!b3) || k<b3->y1 ) b3=box2; }
-          if(box2->x0>3*pp->x/4){ ir++;mr+=y;if ( (!b4) || k<b4->y1 ) b4=box2; }
-        }
-      } end_for_each(&(job->res.boxlist));
-      // b3 near b4 gives large error
-      if(b3!=NULL && b4!=NULL){
-         ml/=il; mr/=ir;
-         if( b3->y1>ml-my/4 && b4->y1>mr-my/4 ) // prevent misinterpretations
-         dy = (b4->y1 - b3->y1)*pp->x/(b4->x1 - b3->x0); 
-         // This is a sample, which works bad without next lines!
-         if( abs((b4->y0 - b3->y0)*pp->x/(b4->x1 - b3->x0))<abs(dy) )
-         dy = (b4->y0 - b3->y0)*pp->x/(b4->x1 - b3->x0); 
-      }
-    }
-    JOB->res.lines.dy=dy;
-    JOB->res.lines.ly=y2;
-    if(JOB->cfg.verbose)fprintf(stderr," - at crosses=%3d dy=%d\n",i,dy);
-  }
+  /* ToDo: normalize to 2^10 bit (square fits to 32 it) */
+  JOB->res.lines.dx=rx;
+  JOB->res.lines.dy=ry;
   return 0;
 }
 
@@ -699,11 +694,11 @@ int detect_rotation_angle(job_t *job){
 int detect_text_lines(pix * pp, int mo) {
 
   if (JOB->cfg.verbose)
-    fprintf(stderr, "# scanning lines\n# ... ");	// or read from outside???
+    fprintf(stderr, "# detect.c detect_text_lines (vvv=16 for more info) ");
   if (mo & 4){
-    if (JOB->cfg.verbose) fprintf(stderr, "(zoning) ");
+    if (JOB->cfg.verbose) fprintf(stderr, "# zoning\n# ... ");
     detect_lines2(pp, 0, 0, pp->x, pp->y, 0);	// later replaced by better algo
-  } else
+  } else 
     detect_lines1(pp, 0, 0, pp->x, pp->y);	// old algo
 
   if(JOB->cfg.verbose) fprintf(stderr,"\n");
@@ -722,19 +717,22 @@ int adjust_text_lines(pix * pp, int mo) {
 
   if ((l=JOB->res.lines.num)<2) return 0;  // ???
   if (JOB->cfg.verbose)
-    fprintf(stderr, "# adjust lines ");
+    fprintf(stderr, "# adjust text lines ");
   m=(int *)malloc(l*16*sizeof(int));
   if (!m) { fprintf(stderr," malloc failed\n"); return 0;}
   for (i=0;i<16*l;i++) m[i]=0; /* initialize */
   dy=JOB->res.lines.dy;  /* tan(alpha) of skewing */
-  dx=JOB->src.p.x;       /* width of image */
+  dx=JOB->res.lines.dx;  /* old: width of image */
   // js: later skewing is replaced by one transformation of vectorized image 
 
+  if (dx)
   for_each_data(&(JOB->res.boxlist)) {
     box2 = (struct box *)list_get_current(&(JOB->res.boxlist));
     if (box2->line<=0) continue;
     if (box2->num_ac<1) continue;
     if (box2->wac[0]<95) continue;
+    if (box2->m2==0 || box2->y1<box2->m2) continue; // char outside line
+    if (box2->m3==4 || box2->y0>box2->m3) continue; // char outside line
     y0=box2->y0-((box2->x1)*dy/dx); /* corrected by page skewing */
     y1=box2->y1-((box2->x1)*dy/dx);
     if (strchr("aemnr",(char)box2->tac[0])) {  // cC vV sS oO ... is unsure!
@@ -763,8 +761,8 @@ int adjust_text_lines(pix * pp, int mo) {
     }
   } end_for_each(&(JOB->res.boxlist));
   
-  diff=0;
   for (i=1;i<l;i++) {
+    diff=0; // show diff per line
     if (m[i*16+4]) diff+=abs(JOB->res.lines.m1[i]-m[i*16+0]/m[i*16+4]);
     if (m[i*16+5]) diff+=abs(JOB->res.lines.m2[i]-m[i*16+1]/m[i*16+5]);
     if (m[i*16+6]) diff+=abs(JOB->res.lines.m3[i]-m[i*16+2]/m[i*16+6]);
@@ -799,7 +797,7 @@ int adjust_text_lines(pix * pp, int mo) {
     if (JOB->res.lines.m4[i]<=JOB->res.lines.m3[i])
         JOB->res.lines.m4[i]= JOB->res.lines.m3[i]+1; /* 4x6 */
                    
-    if (JOB->cfg.verbose & 16)
+    if (JOB->cfg.verbose & 17)
       fprintf(stderr, "\n#  line= %3d m= %4d %+3d %+3d %+3d "
                          " n= %2d %2d %2d %2d  w= %3d diff= %d",
 	i, JOB->res.lines.m1[i],
@@ -811,6 +809,7 @@ int adjust_text_lines(pix * pp, int mo) {
   }
   diff=0; // count adjusted chars
 #if 1
+  if (dx)
   for_each_data(&(JOB->res.boxlist)) {
     box2 = (struct box *)list_get_current(&(JOB->res.boxlist));
     if (box2->line<=0) continue;
@@ -823,15 +822,16 @@ int adjust_text_lines(pix * pp, int mo) {
                     -JOB->res.lines.m1[box2->line]) box2->line=0;
     /* do adjustments */
     if (box2->num_ac>0
-     && strchr("cCoOpPsSuUvVwWxXyYzZ",(char)box2->tac[0])) {
+     && box2->num_ac > 31 && box2->tac[0] < 127 /* islower(>256) may SIGSEGV */
+     && strchr("cCoOpPsSuUvVwWxXyYzZ",(char)box2->tac[0])) { // no_wchar
       if (box2->y0-((box2->x1)*dy/dx)
        < (JOB->res.lines.m1[box2->line]+JOB->res.lines.m2[box2->line])/2
       && islower(box2->tac[0])
-      ) { setac(box2,toupper(box2->tac[0]),(box2->wac[0]+101)/2); diff++; }
+      ) { setac(box2,toupper((char)box2->tac[0]),(box2->wac[0]+101)/2); diff++; }
       if (box2->y0-((box2->x1)*dy/dx)
        > (JOB->res.lines.m1[box2->line]+JOB->res.lines.m2[box2->line]+1)/2
       && isupper(box2->tac[0])
-      ){ setac(box2,tolower(box2->tac[0]),(box2->wac[0]+101)/2); diff++; }
+      ){ setac(box2,tolower((char)box2->tac[0]),(box2->wac[0]+101)/2); diff++; }
     }
     box2->m1=JOB->res.lines.m1[box2->line]+((box2->x1)*dy/dx);
     box2->m2=JOB->res.lines.m2[box2->line]+((box2->x1)*dy/dx);
@@ -841,7 +841,7 @@ int adjust_text_lines(pix * pp, int mo) {
 #endif
 
   free(m);
-  if(JOB->cfg.verbose) fprintf(stderr," changed= %d\n",diff);
+  if(JOB->cfg.verbose) fprintf(stderr,"\n#  changed_chars= %d\n",diff);
   return(diff);
 }
 
@@ -894,15 +894,17 @@ int detect_pictures(job_t *job) {
   struct box *box2, *box4;
 
   if ( job->res.numC == 0 ) {
-    if (job->cfg.verbose) fprintf(stderr, "# detect pictures, numC=0 ???\n");
+    if (job->cfg.verbose) fprintf(stderr,
+      "# detect.c L%d Warning: numC=0\n", __LINE__);
    return -1;
   }
   /* ToDo: set Y to uppercase mean value? */
   job->res.avY = (job->res.sumY+job->res.numC/2) / job->res.numC;
   job->res.avX = (job->res.sumX+job->res.numC/2) / job->res.numC;
+  /* ToDo: two highest volumes? crosses, on extreme volume + on border */
   if (job->cfg.verbose)
-    fprintf(stderr, "# detect pictures, frames, noAlphas, mXmY= %d %d ... ",
-  	    job->res.avX, job->res.avY);
+    fprintf(stderr, "# detect.c L%d pictures, frames, mXmY= %d %d ... ",
+  	    __LINE__, job->res.avX, job->res.avY);
   for_each_data(&(job->res.boxlist)) {
     box2 = (struct box *)list_get_current(&(job->res.boxlist));
     if (box2->c == PICTURE) continue;
