@@ -1,6 +1,6 @@
 /*
 This is a Optical-Character-Recognition program
-Copyright (C) 2000-2006  Joerg Schulenburg
+Copyright (C) 2000-2007  Joerg Schulenburg
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -41,12 +41,15 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "pcx.h"
 #include "ocr0.h"		/* only_numbers */
 #include "progress.h"
-
-/* the following line is used to name the package (make dist) */
-#define version_string "0.44"
+#include "version.h"
 
 static void out_version(int v) {
-  fprintf(stderr, " Optical Character Recognition --- gocr " version_string "\n");
+  fprintf(stderr, " Optical Character Recognition --- gocr "
+          version_string " " release_string "\n"
+          " Copyright (C) 2001-2007 Joerg Schulenburg\n"
+          " released under the GNU General Public License\n");
+  /* as recommended, (c) and license should be part of the binary */
+  /* no email because of SPAM, see README for contacting the author */
   if (v)
     fprintf(stderr, " use option -h for help\n");
   if (v & 2)
@@ -237,7 +240,6 @@ static void mark_start(job_t *job) {
   assert(job);
 
   if (job->cfg.verbose) {
-    fprintf(stderr, "#");
     out_version(0);
     /* insert some helpful info for support */
     fprintf(stderr, "# compiled: " __DATE__ );
@@ -249,7 +251,7 @@ static void mark_start(job_t *job) {
 #endif
 #if defined(__linux)
     fprintf(stderr, " linux");
-#else defined(__unix)
+#elif defined(__unix)
     fprintf(stderr, " unix");
 #endif
 #if defined(__WIN32) || defined(__WIN32__)
@@ -263,9 +265,10 @@ static void mark_start(job_t *job) {
 #endif
     fprintf(stderr, "\n");
     fprintf(stderr,
-	    "# options are: -l %d -s %d -v %d -c %s -m %d -d %d -n %d -a %d\n",
-	    job->cfg.cs, job->cfg.spc, job->cfg.verbose, job->cfg.lc, job->cfg.mode, 
-            job->cfg.dust_size, job->cfg.only_numbers, job->cfg.certainty);
+      "# options are: -l %d -s %d -v %d -c %s -m %d -d %d -n %d -a %d -C \"%s\"\n",
+      job->cfg.cs, job->cfg.spc, job->cfg.verbose, job->cfg.lc, job->cfg.mode, 
+      job->cfg.dust_size, job->cfg.only_numbers, job->cfg.certainty,
+      job->cfg.cfilter);
     fprintf(stderr, "# file: %s\n", job->src.fname);
 #ifdef USE_UNICODE
     fprintf(stderr,"# using unicode\n");
