@@ -1,6 +1,6 @@
 /*
 This is a Optical-Character-Recognition program
-Copyright (C) 2000-2006 Joerg Schulenburg
+Copyright (C) 2000-2007 Joerg Schulenburg
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -59,7 +59,7 @@ char *append_to_line(char *buffer, const char *s1, int *len) {
   char *temp;
   int slen=0, alen;
   if( s1==NULL || s1[0] == 0 ){
-    fprintf(stderr,"\n#BUG: appending 0 to a line makes no sence!");
+    fprintf(stderr,"\n#BUG: appending 0 to a line makes no sense!");
     return buffer;
   }
   if ( *len>0 ) slen= strlen(buffer);  // used buffer
@@ -97,25 +97,22 @@ int calc_median_gap(struct tlines * lines) {
  *  indent.
  *
  *  boxlist -- list of character boxes.
- *  page_width -- width of page as a whole.  Used in
- *     calculating rotation.
- *  dy -- change in y coordinates of the line used to 
- *     determine rotation angle, across the length of the
- *     page.
+ *  dx, dy -- rotation angle as vector
  */
-int get_least_line_indent(List * boxlist, int page_width, int dy) {
+int get_least_line_indent(List * boxlist, int dx, int dy) {
     int min_indent = INT_MAX;
     int adjusted_indent;
     struct box * box2;
     if (JOB->cfg.verbose)
-      fprintf(stderr, "get_least_line_indent: page_width %d, dy %d\n",
-              page_width, dy);
+      fprintf(stderr, "get_least_line_indent: rot.vector dxdy %d %d\n",
+              dx, dy);
     for_each_data(boxlist) {
         box2 = (struct box *)list_get_current(boxlist);
         /* if num == -1, indicates this is a space or newline box,
          * inserted in list_insert_spaces. */
         if (box2->num != -1) {
-            adjusted_indent = box2->x0 + box2->y0 * dy / page_width;
+            adjusted_indent = box2->x0;
+            if (dx) adjusted_indent += box2->y0 * dy / dx;
             if (adjusted_indent < min_indent) {
                 min_indent = adjusted_indent;
                 if (dy!=0 && JOB->cfg.verbose)
@@ -181,7 +178,8 @@ void store_boxtree_lines(int mo) {
   }
 
   // Will be subtracted as base_indent to avoid negativ indent.
-  left_margin = get_least_line_indent(&JOB->res.boxlist, JOB->src.p.x, 
+  left_margin = get_least_line_indent(&JOB->res.boxlist,
+          JOB->res.lines.dx, 
           JOB->res.lines.dy);
 
   if (JOB->cfg.out_format==XML) { /* subject of change */
@@ -199,6 +197,8 @@ void store_boxtree_lines(int mo) {
     box2 = (struct box *)list_get_current(&(JOB->res.boxlist));
     line = box2->line;
     line_info = JOB->res.lines;
+    /* reset the output char if certainty is below the limit v0.44 */
+    if (box2->num_ac && box2->wac[0]<JOB->cfg.certainty) box2->c=UNKNOWN;
     if (line!=oldline) {
       if (JOB->cfg.out_format==XML && oldline>-1) { /* subject of change */
         buffer=append_to_line(buffer,"</line>\n",&len);
@@ -248,7 +248,7 @@ void store_boxtree_lines(int mo) {
         } else
         for (i = (box2->x1 - box2->x0) / (2 * JOB->res.avX) + 1; i > 0; i--) {
           buffer=append_to_line(buffer," ",&len);
-          j++;
+          j++; /* number of chars in line */
         }
       }
     }
@@ -256,8 +256,8 @@ void store_boxtree_lines(int mo) {
       if (j==0 && JOB->res.avX) /* first char in new line? */ {
         int indent = box2->x0 - JOB->res.lines.x0[box2->line];
         /* correct for angle of page as a whole. */
-        indent += box2->y0 * JOB->res.lines.dy
-          / JOB->src.p.x;
+        if (JOB->res.lines.dx)
+          indent += box2->y0 * JOB->res.lines.dy / JOB->res.lines.dx;
         /* subtract the base margin. */
         indent -= left_margin;
         if (JOB->cfg.out_format==XML) { /* subject of change */
@@ -277,7 +277,7 @@ void store_boxtree_lines(int mo) {
         sprintf(s1," <box x=\"%d\" y=\"%d\" dx=\"%d\" dy=\"%d\" value=\"",
                 box2->x0,box2->y0,box2->x1-box2->x0+1,box2->y1-box2->y0+1);
         buffer=append_to_line(buffer,s1,&len);
-        if (box2->num_ac>1) { /* output alist */
+        if (box2->num_ac>1) { /* ToDo: output a list of alternatives */
         }
       }
       if (box2->c != UNKNOWN && box2->c!=0) {
@@ -285,15 +285,19 @@ void store_boxtree_lines(int mo) {
           append_to_line(buffer,decode(box2->c,JOB->cfg.out_format),&len);
         if (box2->c >  ' ' &&
             box2->c <= 'z') i2++; /* count non-space chars */
-      } else {
+      } else { /* c == UNKNOWN */
         wchar_t cc; cc=box2->c;
         if (box2->num_ac>0 && box2->tas[0]
                 && (JOB->cfg.out_format!=XML || box2->tas[0][0]!='<')) {
+/*
           buffer=append_to_line(buffer,box2->tas[0],&len);
           j+=strlen(box2->tas[0]);
+*/
         } else {
-          buffer=
+          /*
+           buffer=
             append_to_line(buffer,decode(cc,JOB->cfg.out_format),&len);
+          */
         }
       }
       if (JOB->cfg.out_format==XML) {
@@ -310,8 +314,12 @@ void store_boxtree_lines(int mo) {
           if (box2->num_ac>1)
             buffer=append_to_line(buffer,"\" achars=\"",&len);
           for (i1=1;i1<box2->num_ac;i1++) {
-            buffer=append_to_line(buffer,
+            if (box2->tas[i1] && box2->tas[i1][0]!='<')
+              buffer=append_to_line(buffer,box2->tas[i1],&len);
+            else
+              buffer=append_to_line(buffer,
                  decode(box2->tac[i1],JOB->cfg.out_format),&len);
+            // ToDo: add tas[] (achars->avalues or alternate_strings?
             if (i1+1<box2->num_ac) buffer=append_to_line(buffer,",",&len);
           }
         }
@@ -324,7 +332,7 @@ void store_boxtree_lines(int mo) {
           j+=strlen(box2->tas[0]);
         }
       }
-      j++;
+      j++; /* number of chars in line */
     }
     i++;
   } end_for_each(&(JOB->res.boxlist));
