@@ -256,43 +256,58 @@ static int fread_num(char *buf, int bps, FILE *f1) {
   return 0;
 }
 
-int readpgm(char *name,pix *p,int vvv){	// raw-ppm
-  static char c1,c2;
+/*
+ * read image file, used to read the OCR-image and database images,
+ * image file can be PBM/PGM/PPM in RAW or TEXT
+ *   name: filename of image (input)
+ *   p:    pointer where to store the loaded image (input)
+ *   vvv:  verbose mode (input)
+ *   return: 0=ok, 1=further image follows (multiple image), -1 on error
+ * this is the fall back routine if libpnm cant be used
+ */
+int readpgm( char *name, pix *p, int vvv){
+  static char c1, c2;           /* magic bytes, file type */
   static char *pip;             // static to survive multiple calls
   int  nx,ny,nc,mod,i,j;	// buffer
-  static FILE *f1=NULL;
+  static FILE *f1=NULL;         // trigger read new file or multi image file
   unsigned char *pic;
   char buf[512];
   int lx, ly, dx;
   int bps=1; /* bytes per sample (0..255..65535...) */
 
-  if (!f1) {  /* first of multiple image */
+  if (!f1) {  /* first of multiple image, on MultipleImageFiles c1 was read */
     pip=NULL;
-    if(name[0]=='-' && name[1]==0){
+    if (name[0]=='-' && name[1]==0) {
       f1=stdin;  /* is this correct ??? */
     } else {
       pip=testsuffix(name);
-      if(!pip){
-        f1=fopen(name,"rb"); if(!f1)F1("opening file %s",name);
+      if (!pip) {
+        f1=fopen(name,"rb"); if (!f1) F1("opening file %s",name);
       } else {
         sprintf(buf,"%s %s",pip,name); /* ToDo: how to prevent OVL ? */
-        if(vvv){ fprintf(stderr,"# popen( %s )\n",buf); }
+        if (vvv) { fprintf(stderr,"# popen( %s )\n",buf); }
 #ifdef HAVE_POPEN
         f1=popen(buf,"r");
 #else
         F0("only PNM files supported (compiled without HAVE_POPEN)");
 #endif
-        if(!f1)F1("opening pipe %s",buf);
+        if (!f1) F1("opening pipe %s",buf);
       }
     }
-    c1=fgetc(f1); if (feof(f1)) { E0("read magic1"); return -1; }
+    c1=fgetc(f1); if (feof(f1)) { E0("unexpected EOF"); return -1; }
   }
-  c2=fgetc(f1); if (feof(f1)) { E0("read magic2"); return -1; }
+  c2=fgetc(f1);   if (feof(f1)) { E0("unexpected EOF"); return -1; }
   // check the first two bytes of the PNM file 
   //         PBM   PGM   PPM
   //    TXT   P1    P2    P3
   //    RAW   P4    P5    P6
-  if (c1!='P' || c2 <'1' || c2 >'6') F0("no PNM (P1..P6)"); // PNM-TXT/RAW
+  if (c1!='P' || c2 <'1' || c2 >'6') { 
+    fprintf(stderr,"\nread-PNM-error: file descriptor is %08x,"
+                   " position %ld", f1, ftell(f1));
+    fprintf(stderr,"\nread-PNM-error: bad magic bytes, expect 0x50 0x3[1-6]"
+                   " but got 0x%02x 0x%02x", 255&c1, 255&c2);
+    if (f1) close(f1); f1=NULL; return(-1);
+  }
   nx=ny=nc=0; if (c2=='4' || c2=='1') nc=1;
   for(mod=0;((c2=='5' || c2=='2') && (mod&7)<6)
         ||  ((c2=='6' || c2=='3') && (mod&7)<6)		
@@ -370,17 +385,16 @@ int readpgm(char *name,pix *p,int vvv){	// raw-ppm
   }
   p->p=pic;  p->x=nx;  p->y=ny; p->bpp=1;
   if (vvv) fprintf(stderr,"\n");  
-  c1=fgetc(f1); /* needed to trigger feof() */
-  if (feof(f1)) {  /* dont hit if at end and no further read */
+  c1=0; c1=fgetc(f1); /* needed to trigger feof() */
+  if (feof(f1) || c1!='P') {  /* EOF ^Z or not 'P' -> single image */
     if (vvv) fprintf(stderr,"# PNM EOF\n");
-    if(name[0]=='-' && name[1]==0){
-      f1=NULL;
-    } else {
+    if(name[0]!='-' || name[1]!=0){ /* do not close stdin */
       if(!pip) fclose(f1);
 #ifdef HAVE_POPEN
       else     pclose(f1);	/* close pipe (Jul00) */
 #endif
     }
+    f1=NULL;  /* set file is closed flag */
     return 0;
   }
   return 1; /* multiple image = concatenated pnm's */
