@@ -51,7 +51,7 @@ int remove_dust( job_t *job ){
   pix *pp = &job->src.p;
   int i1,i,j,x,y,x0,x1,y0,y1,nC,sX,sY,sP, cs,vvv=job->cfg.verbose;
   struct box *box2;
-#define HISTSIZE  220   /* histogramm */
+#define HISTSIZE  220   /* histogramm size */
   int histo[HISTSIZE];
   cs=job->cfg.cs; sP=sX=sY=nC=0;
   /*
@@ -85,7 +85,8 @@ int remove_dust( job_t *job ){
      *  does not work for background pattern!
      */
     job->cfg.dust_size = (  ( sX/nC ) * ( sY/nC ) + 16) / 32;
-    if (vvv) fprintf(stderr, "# dust size detection, vol num"
+    if (vvv) fprintf(stderr, "# remove.c remove_dust(): ");
+    if (vvv) fprintf(stderr, "\n# dust size detection, vol num"
       " #obj=%d maxDust=%d mpixel= %3d mxy= %2d %2d",
       nC, job->cfg.dust_size, sP/nC, sX/nC, sY/nC);
     /* we assume that for random dust applies histo[i+1]<histo[i] */
@@ -93,7 +94,7 @@ int remove_dust( job_t *job ){
       if (vvv) fprintf(stderr,"\n# dust size histogram %3d %5d",i,histo[i]);
       if (histo[i]>=nC) continue; /* v0.42 lot of pixels -> bg pattern < 3 */
       if (i>=job->cfg.dust_size) break;   /* maximum = mean size / 32 */
-      if (histo[i+1]==0) break;           /* bad statistic */
+      if (histo[i/*+1*/]==0) break;           /* bad statistic */
       if ((histo[i+2]+histo[i+3])
         >=(histo[i]  +histo[i+1])) break;  /* no noise, but to late? */
       if ( histo[i-1] > 1024*histo[i] &&
@@ -323,6 +324,7 @@ int remove_pictures( job_t *job){
   for_each_data(&(job->res.boxlist)) {
     box2 = (struct box *)list_get_current(&(job->res.boxlist));
     if (box2->c==PICTURE
+     && box2->num_ac==0	 /* dont remove barcodes */
      && box2->x1-box2->x0+1>box2->p->x/2  /* big table? */
      && box2->y1-box2->y0+1>box2->p->y/2 ){ j=0;
       /* count boxes nested with the picture */
